@@ -1,16 +1,16 @@
 /*  Evolution RSS Reader Plugin
- *  Copyright (C) 2007-2009 Lucian Langa <cooly@gnome.eu.org>
- *         
+ *  Copyright (C) 2007-2010 Lucian Langa <cooly@gnome.eu.org>
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or 
+ *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
- *                     
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
- *                                             
+ *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -33,18 +33,17 @@
 #include <libedataserver/e-proxy.h>
 #endif
 
+extern int rss_verbose_debug;
+
 #include "network.h"
 #include "network-soup.h"
 #include "rss.h"
 #include "misc.h"
 
-
 #define USE_PROXY FALSE
 
 #define SS_TIMEOUT 30
 
-#define d(x)
-
 #if LIBSOUP_VERSION > 2024000
 SoupCookieJar *rss_soup_jar = NULL;
 #endif
@@ -63,36 +62,48 @@ typedef struct {
 	gchar *chunk;
 } CallbackInfo;
 
+typedef struct {
+	SoupSession *ss;
+	SoupMessage *sm;
+	gpointer cb2;
+	gpointer cbdata2;
+	gchar *url;
+} STNET;
+
+#define DOWNLOAD_QUEUE_SIZE 15
+guint net_qid = 0;		// net queue dispatcher
+guint net_queue_run_count = 0; //downloads in progress
+
 static void
 #if LIBSOUP_VERSION < 2003000
 got_chunk_blocking_cb(SoupMessage *msg, CallbackInfo *info) {
 #else
 got_chunk_blocking_cb(SoupMessage *msg, SoupBuffer *chunk, CallbackInfo *info) {
 #endif
-    NetStatusProgress progress = {0};
-    const char* clen;
+	NetStatusProgress progress = {0};
+	const char* clen;
 
-    if (info->total == 0) {
+	if (info->total == 0) {
 #if LIBSOUP_VERSION < 2003000
-        clen = soup_message_get_header(msg->response_headers,
-                "Content-length");
+		clen = soup_message_get_header(msg->response_headers,
+			"Content-length");
 #else
-        clen = soup_message_headers_get(msg->response_headers,
-                "Content-length");
+		clen = soup_message_headers_get(msg->response_headers,
+			"Content-length");
 #endif
-        if (!clen)
-            return;
-        info->total = atoi(clen);
-    }
+		if (!clen)
+			return;
+		info->total = atoi(clen);
+	}
 #if LIBSOUP_VERSION < 2003000
-    info->current += msg->response.length;
+	info->current += msg->response.length;
 #else
-    info->current += chunk->length;
+	info->current += chunk->length;
 #endif
 
-    progress.current = info->current;
-    progress.total = info->total;
-    info->user_cb(NET_STATUS_PROGRESS, &progress, info->user_data);
+	progress.current = info->current;
+	progress.total = info->total;
+	info->user_cb(NET_STATUS_PROGRESS, &progress, info->user_data);
 }
 
 static void
@@ -104,14 +115,14 @@ got_chunk_cb(SoupMessage *msg, SoupBuffer *chunk, CallbackInfo *info) {
 
 	NetStatusProgress *progress = NULL;
 	const char* clen;
-	
+
 	if (info->total == 0) {
 #if LIBSOUP_VERSION < 2003000
 		clen = soup_message_get_header(msg->response_headers,
 				"Content-length");
 			return;
 #else
-        	clen = soup_message_headers_get(msg->response_headers,
+		clen = soup_message_headers_get(msg->response_headers,
 				"Content-length");
 #endif
 		if (!clen)
@@ -140,12 +151,16 @@ int net_error_quark(void)
 	return 0;
 }
 
+void unblocking_error (SoupMessage *msg, gpointer user_data);
+
 void
 unblocking_error (SoupMessage *msg, gpointer user_data)
 {
 	g_print("data:%p\n", user_data);
 }
 
+void recv_msg (SoupMessage *msg, gpointer user_data);
+
 void
 recv_msg (SoupMessage *msg, gpointer user_data)
 {
@@ -155,8 +170,8 @@ recv_msg (SoupMessage *msg, gpointer user_data)
 #else
 	response = g_string_new_len(msg->response_body->data, msg->response_body->length);
 #endif
-	d(g_print("got it!\n"));
-	d(g_print("res:[%s]\n", response->str));
+	d("got it!\n");
+	d("res:[%s]\n", response->str);
 }
 
 static gboolean
@@ -171,6 +186,8 @@ remove_if_match (gpointer key, gpointer value, gpointer user_data)
 		return FALSE;
 }
 
+void construct_abort(gpointer key, gpointer value, gpointer user_data);
+
 void
 construct_abort(gpointer key, gpointer value, gpointer user_data)
 {
@@ -180,7 +197,7 @@ construct_abort(gpointer key, gpointer value, gpointer user_data)
 static void
 unblock_free (gpointer user_data, GObject *ex_msg)
 {
-	d(g_print("weak ref - trying to free object\n"));
+	d("weak ref - trying to free object\n");
 	g_hash_table_remove(rf->session, user_data);
 	g_hash_table_destroy(rf->abort_session);
 	rf->abort_session = g_hash_table_new(g_direct_hash, g_direct_equal);
@@ -201,7 +218,7 @@ proxy_init(void)
 {
 	EProxy *proxy;
 	proxy = e_proxy_new ();
-        e_proxy_setup_proxy (proxy);
+	e_proxy_setup_proxy (proxy);
 	return proxy;
 }
 
@@ -219,18 +236,22 @@ proxify_webkit_session(EProxy *proxy, gchar *uri)
 		if (e_proxy_require_proxy_for_uri (proxy, uri)) {
 #if (DATASERVER_VERSION >=2026000)
 			proxy_uri = e_proxy_peek_uri_for (proxy, uri);
-			d(g_print("webkit proxified %s with %s:%d\n", uri, proxy_uri->host, proxy_uri->port));
+			d("webkit proxified %s with %s:%d\n", uri, proxy_uri->host, proxy_uri->port);
 #else
 			g_print("WARN: e_proxy_peek_uri_for() requires evolution-data-server 2.26\n");
 			return;
 #endif
-		} else 
-			d(g_print("webkit no PROXY-%s\n", uri));
+		} else  {
+			d("webkit no PROXY-%s\n", uri);
+		}
 		break;
-		g_object_set (G_OBJECT (webkit_session), SOUP_SESSION_PROXY_URI, proxy_uri, NULL);
+		g_object_set (
+			G_OBJECT (webkit_session),
+			SOUP_SESSION_PROXY_URI, proxy_uri, NULL);
 #ifdef HAVE_LIBSOUP_GNOME
 	case 0:
-		soup_session_add_feature_by_type (webkit_session, SOUP_TYPE_PROXY_RESOLVER_GNOME);
+		soup_session_add_feature_by_type (
+			webkit_session, SOUP_TYPE_PROXY_RESOLVER_GNOME);
 		break;
 #endif
 	}
@@ -242,7 +263,8 @@ void
 proxify_session(EProxy *proxy, SoupSession *session, gchar *uri)
 {
 	SoupURI *proxy_uri = NULL;
-	gint ptype = gconf_client_get_int (rss_gconf, KEY_GCONF_EVO_PROXY_TYPE, NULL);
+	gint ptype = gconf_client_get_int (
+			rss_gconf, KEY_GCONF_EVO_PROXY_TYPE, NULL);
 
 	switch (ptype) {
 #ifndef HAVE_LIBSOUP_GNOME
@@ -256,16 +278,22 @@ proxify_session(EProxy *proxy, SoupSession *session, gchar *uri)
 			g_print("WARN: e_proxy_peek_uri_for() requires evolution-data-server 2.26\n");
 			return;
 #endif
-			if (proxy_uri)
-				d(g_print("proxified %s with %s:%d\n", uri, proxy_uri->host, proxy_uri->port));
-		} else 
-			d(g_print("no PROXY-%s\n", uri));
-		g_object_set (G_OBJECT (session), SOUP_SESSION_PROXY_URI, proxy_uri, NULL);
+			if (proxy_uri) {
+				d("proxified %s with %s:%d\n", uri, proxy_uri->host, proxy_uri->port);
+			}
+		} else {
+			d("no PROXY-%s\n", uri);
+		}
+		g_object_set (
+			G_OBJECT (session),
+			SOUP_SESSION_PROXY_URI,
+			proxy_uri, NULL);
 		break;
 
 #ifdef HAVE_LIBSOUP_GNOME
 	case 0:
-			soup_session_add_feature_by_type (session, SOUP_TYPE_PROXY_RESOLVER_GNOME);
+			soup_session_add_feature_by_type (
+				session, SOUP_TYPE_PROXY_RESOLVER_GNOME);
 		break;
 #endif
 	}
@@ -276,30 +304,34 @@ proxify_session(EProxy *proxy, SoupSession *session, gchar *uri)
 guint
 read_up(gpointer data)
 {
+	FILE *fr;
 	char rfeed[512];
 	guint res = 0;
-	
+	gchar *tmp, *buf, *feed_dir, *feed_name;
+
 	if (NULL != g_hash_table_lookup(rf->hruser, data))
 		return 1;
 
-	gchar *tmp = gen_md5(data);
-	gchar *buf = g_strconcat(tmp, ".rec", NULL);
+	tmp = gen_md5(data);
+	buf = g_strconcat(tmp, ".rec", NULL);
 	g_free(tmp);
 
-	gchar *feed_dir = rss_component_peek_base_directory(mail_component_peek());
+	feed_dir = rss_component_peek_base_directory();
 	if (!g_file_test(feed_dir, G_FILE_TEST_EXISTS))
-            g_mkdir_with_parents (feed_dir, 0755);
-
-	gchar *feed_name = g_strdup_printf("%s/%s", feed_dir, buf);
+		g_mkdir_with_parents (feed_dir, 0755);
+	feed_name = g_build_path(G_DIR_SEPARATOR_S, feed_dir, buf, NULL);
 	g_free(feed_dir);
+	d("reading auth info:%s\n", feed_name);
 
-	FILE *fr = fopen(feed_name, "r");
+	fr = fopen(feed_name, "r");
 	if (fr) {
-        	fgets(rfeed, 511, fr);
-        	g_hash_table_insert(rf->hruser, data, g_strstrip(g_strdup(rfeed)));
-        	fgets(rfeed, 511, fr);
-        	g_hash_table_insert(rf->hrpass, data, g_strstrip(g_strdup(rfeed)));
-        	fclose(fr);
+		fgets(rfeed, 511, fr);
+		g_hash_table_insert(
+			rf->hruser, data, g_strstrip(g_strdup(rfeed)));
+		fgets(rfeed, 511, fr);
+		g_hash_table_insert(
+			rf->hrpass, data, g_strstrip(g_strdup(rfeed)));
+		fclose(fr);
 		res = 1;
 	}
 	g_free(feed_name);
@@ -310,27 +342,28 @@ read_up(gpointer data)
 guint
 save_up(gpointer data)
 {
+	FILE *fr;
+	guint res = 0;
+	gchar *feed_dir, *feed_name, *user, *pass;
 	gchar *tmp = gen_md5(data);
 	gchar *buf = g_strconcat(tmp, ".rec", NULL);
 	g_free(tmp);
-	guint res = 0;
 
-	gchar *feed_dir = rss_component_peek_base_directory(mail_component_peek());
+	feed_dir = rss_component_peek_base_directory();
 	if (!g_file_test(feed_dir, G_FILE_TEST_EXISTS))
-            g_mkdir_with_parents (feed_dir, 0755);
-
-	gchar *feed_name = g_strdup_printf("%s/%s", feed_dir, buf);
+		g_mkdir_with_parents (feed_dir, 0755);
+	feed_name = g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s", feed_dir, buf);
 	g_free(feed_dir);
 
-	FILE *fr = fopen(feed_name, "w+");
+	fr = fopen(feed_name, "w+");
 	if (fr) {
-        	gchar *user = g_hash_table_lookup(rf->hruser, data);
+		user = g_hash_table_lookup(rf->hruser, data);
 			fputs(user, fr);
-	        fputs("\n", fr);
-        	gchar *pass = g_hash_table_lookup(rf->hrpass, data);
-        	fputs(pass, fr);
-        	fclose(fr);
-        	res = 1;
+		fputs("\n", fr);
+		pass = g_hash_table_lookup(rf->hrpass, data);
+		fputs(pass, fr);
+		fclose(fr);
+		res = 1;
 	}
 	g_free(feed_name);
 	g_free(buf);
@@ -340,14 +373,14 @@ save_up(gpointer data)
 guint
 del_up(gpointer data)
 {
+	gchar *feed_dir, *feed_name;
 	gchar *tmp = gen_md5(data);
 	gchar *buf = g_strconcat(tmp, ".rec", NULL);
 	g_free(tmp);
-	gchar *feed_dir = rss_component_peek_base_directory(mail_component_peek());
+	feed_dir = rss_component_peek_base_directory();
 	if (!g_file_test(feed_dir, G_FILE_TEST_EXISTS))
-            g_mkdir_with_parents (feed_dir, 0755);
-
-	gchar *feed_name = g_strdup_printf("%s/%s", feed_dir, buf);
+		g_mkdir_with_parents (feed_dir, 0755);
+	feed_name = g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s", feed_dir, buf);
 	g_free(feed_dir);
 	unlink(feed_name);
 	g_free(feed_name);
@@ -358,23 +391,26 @@ del_up(gpointer data)
 static void
 #if LIBSOUP_VERSION < 2003000
 authenticate (SoupSession *session,
-        SoupMessage *msg,
-        const char *auth_type,
-        const char *auth_realm,
-        char **username,
-        char **password,
-        gpointer data)
+		SoupMessage *msg,
+		const char *auth_type,
+		const char *auth_realm,
+		char **username,
+		char **password,
+		gpointer data)
 #else
 authenticate (SoupSession *session,
-	SoupMessage *msg,
-        SoupAuth *auth,
-	gboolean retrying,
-	gpointer data)
+		SoupMessage *msg,
+		SoupAuth *auth,
+		gboolean retrying,
+		gpointer data)
 #endif
 {
+	SoupURI *proxy_uri;
+	gchar *user, *pass;
+	RSS_AUTH *auth_info = g_new0(RSS_AUTH, 1);
+
 	if (msg->status_code == SOUP_STATUS_PROXY_UNAUTHORIZED) {
 		g_print("proxy:%d\n", soup_auth_is_for_proxy(auth));
-	SoupURI *proxy_uri;
 	g_object_get (G_OBJECT(session),
 				"proxy-uri", &proxy_uri,
 				NULL);
@@ -383,9 +419,10 @@ authenticate (SoupSession *session,
 	return;
 	}
 
-	gchar *user = g_hash_table_lookup(rf->hruser, data);
-	gchar *pass = g_hash_table_lookup(rf->hrpass, data);
-	
+	user = g_hash_table_lookup(rf->hruser, data);
+	pass = g_hash_table_lookup(rf->hrpass, data);
+	d("data:%s, user:%s, pass:%s\n", (gchar *)data, user, pass);
+
 	if (user && pass) {
 #if LIBSOUP_VERSION < 2003000
 		*username = g_strdup(user);
@@ -399,23 +436,36 @@ authenticate (SoupSession *session,
 	}
 #endif
 	} else {
+		read_up(data);
+		user = g_hash_table_lookup(rf->hruser, data);
+		pass = g_hash_table_lookup(rf->hrpass, data);
+		if (user && pass) {
+#if LIBSOUP_VERSION < 2003000
+			*username = g_strdup(user);
+			*password = g_strdup(pass);
+#else
+			if (!retrying)
+				soup_auth_authenticate (auth, user, pass);
+			return;
+#endif
+		}
 		//we test for autofetching in progresss because it seems
 		//preety annoying to pop the authentication popup in front
 		//of the user every time feeds are automatically fetched
-		if (!read_up(data) && !rf->autoupdate) {
-			//we will continue after user has made a decision on 
+		if (!rf->autoupdate) {
+			//we will continue after user has made a decision on
 			//web auth dialog
 			//Bug 522147 – need to be able to pause synchronous I/O
 authpop:		if (G_OBJECT_TYPE(session) == SOUP_TYPE_SESSION_ASYNC) {
 				soup_session_pause_message(session, msg);
 			}
-			RSS_AUTH *auth_info = g_new0(RSS_AUTH, 1);
 			auth_info->url = data;
 			auth_info->soup_auth = auth;
 			auth_info->retrying = retrying;
 			auth_info->session = session;
 			auth_info->message = msg;
 			web_auth_dialog(auth_info);
+			return;
 		}
 	}
 }
@@ -423,12 +473,12 @@ authpop:		if (G_OBJECT_TYPE(session) == SOUP_TYPE_SESSION_ASYNC) {
 #if LIBSOUP_VERSION < 2003000
 static void
 reauthenticate (SoupSession *session,
-        SoupMessage *msg,
-        const char *auth_type,
-        const char *auth_realm,
-        char **username,
-        char **password,
-        gpointer data)
+		SoupMessage *msg,
+		const char *auth_type,
+		const char *auth_realm,
+		char **username,
+		char **password,
+		gpointer data)
 {
 	if (rf->soup_auth_retry) {
 		//means we're already tested once and probably
@@ -439,12 +489,14 @@ reauthenticate (SoupSession *session,
 		} else {
 			rf->soup_auth_retry = TRUE;
 		}
-        	*username = g_strdup(g_hash_table_lookup(rf->hruser, data));
-        	*password = g_strdup(g_hash_table_lookup(rf->hrpass, data));
+		*username = g_strdup(g_hash_table_lookup(rf->hruser, data));
+		*password = g_strdup(g_hash_table_lookup(rf->hrpass, data));
 	}
 }
 #endif
 
+guint net_get_status(const char *url, GError **err);
+
 guint
 net_get_status(const char *url, GError **err)
 {
@@ -457,16 +509,18 @@ net_get_status(const char *url, GError **err)
 	guint response = 0;
 	SoupSession *soup_sess = NULL;
 	GSList *headers = NULL;
+	gchar *agstr;
 
 	if (!rf->b_session)
-		rf->b_session = soup_sess = 
-			soup_session_sync_new_with_options(SOUP_SESSION_TIMEOUT, SS_TIMEOUT, NULL);		
+		rf->b_session = soup_sess =
+			soup_session_sync_new_with_options(
+				SOUP_SESSION_TIMEOUT, SS_TIMEOUT, NULL);
 	else
 		soup_sess = rf->b_session;
 
 	req = soup_message_new(SOUP_METHOD_GET, url);
 	if (!req) {
-		g_set_error(err, NET_ERROR, NET_ERROR_GENERIC,
+		g_set_error(err, NET_ERROR, NET_ERROR_GENERIC, "%s",
 				soup_status_get_phrase(2));			//invalid url
 		goto out;
 	}
@@ -477,20 +531,24 @@ net_get_status(const char *url, GError **err)
 		char *colonpos = strchr(header, ':');
 		*colonpos = 0;
 #if LIBSOUP_VERSION < 2003000
-		soup_message_add_header(req->request_headers, header, colonpos+1);
+		soup_message_add_header(
+			req->request_headers, header, colonpos+1);
 #else
-		soup_message_headers_append(req->request_headers, header, colonpos+1);
+		soup_message_headers_append(
+			req->request_headers, header, colonpos+1);
 #endif
 		*colonpos = ':';
 	}
-	gchar *agstr = g_strdup_printf("Evolution/%s; Evolution-RSS/%s",
+	agstr = g_strdup_printf("Evolution/%s; Evolution-RSS/%s",
 			EVOLUTION_VERSION_STRING, VERSION);
 #if LIBSOUP_VERSION < 2003000
-	soup_message_add_header (req->request_headers, "User-Agent",
-                                agstr);
+	soup_message_add_header (req->request_headers,
+		"User-Agent",
+		agstr);
 #else
-	soup_message_headers_append (req->request_headers, "User-Agent",
-                                agstr);
+	soup_message_headers_append (req->request_headers,
+		"User-Agent",
+		agstr);
 #endif
 	g_free(agstr);
 
@@ -503,7 +561,7 @@ net_get_status(const char *url, GError **err)
 		soup_session_abort(soup_sess);
 		g_object_unref(soup_sess);
 		rf->b_session = NULL;
-		g_set_error(err, NET_ERROR, NET_ERROR_GENERIC,
+		g_set_error(err, NET_ERROR, NET_ERROR_GENERIC, "%s",
 				soup_status_get_phrase(req->status_code));
 		goto out;
 	}
@@ -512,28 +570,29 @@ out:
 	if (suri) soup_uri_free(suri);
 	response = req->status_code;
 	if (req) g_object_unref(G_OBJECT(req));
-	
+
 	return response;
 }
 
 gboolean
-net_get_unblocking(gchar *url, 
-				NetStatusCallback cb, gpointer data, 
-				gpointer cb2, gpointer cbdata2,
-				guint track,
-				GError **err)
+net_get_unblocking(gchar *url,
+			NetStatusCallback cb, gpointer data,
+			gpointer cb2, gpointer cbdata2,
+			guint track,
+			GError **err)
 {
 	SoupMessage *msg;
 	CallbackInfo *info = NULL;
+	SoupSession *soup_sess;
+	gchar *agstr;
+
+	soup_sess = soup_session_async_new();
 
-	SoupSession *soup_sess = 
-//		soup_session_async_new_with_options(SOUP_SESSION_TIMEOUT, SS_TIMEOUT, NULL);
-		soup_session_async_new();
-			
 
 #if LIBSOUP_VERSION > 2024000
 	if (rss_soup_jar) {
-		soup_session_add_feature(soup_sess, SOUP_SESSION_FEATURE(rss_soup_jar));
+		soup_session_add_feature(
+			soup_sess, SOUP_SESSION_FEATURE(rss_soup_jar));
 	}
 #endif
 
@@ -548,25 +607,17 @@ net_get_unblocking(gchar *url,
 		info->total = 0;
 	}
 
-	if (!rf->session)
-		rf->session = g_hash_table_new(g_direct_hash, g_direct_equal);
-	if (!rf->abort_session)
-		rf->abort_session = g_hash_table_new(g_direct_hash, g_direct_equal);
-	if (!rf->key_session)
-		rf->key_session = g_hash_table_new(g_direct_hash, g_direct_equal);
-
-
 	g_signal_connect (soup_sess, "authenticate",
-            G_CALLBACK (authenticate), (gpointer)url);
+		G_CALLBACK (authenticate), (gpointer)url);
 #if LIBSOUP_VERSION < 2003000
 	g_signal_connect (soup_sess, "reauthenticate",
-            G_CALLBACK (reauthenticate), (gpointer)url);
+		G_CALLBACK (reauthenticate), (gpointer)url);
 #endif
 
 	/* Queue an async HTTP request */
 	msg = soup_message_new ("GET", url);
 	if (!msg) {
-		g_set_error(err, NET_ERROR, NET_ERROR_GENERIC,
+		g_set_error(err, NET_ERROR, NET_ERROR_GENERIC, "%s",
 				soup_status_get_phrase(2));			//invalid url
 		return FALSE;
 	}
@@ -579,14 +630,14 @@ net_get_unblocking(gchar *url,
 		g_hash_table_insert(rf->key_session, data, soup_sess);
 	}
 
-	gchar *agstr = g_strdup_printf("Evolution/%s; Evolution-RSS/%s",
+	agstr = g_strdup_printf("Evolution/%s; Evolution-RSS/%s",
 			EVOLUTION_VERSION_STRING, VERSION);
 #if LIBSOUP_VERSION < 2003000
 	soup_message_add_header (msg->request_headers, "User-Agent",
-                                agstr);
+		agstr);
 #else
 	soup_message_headers_append (msg->request_headers, "User-Agent",
-                                agstr);
+		agstr);
 #endif
 	g_free(agstr);
 
@@ -596,20 +647,150 @@ net_get_unblocking(gchar *url,
 	}
 
 	soup_session_queue_message (soup_sess, msg,
-           cb2, cbdata2);
+		cb2, cbdata2);
 
 ////	g_object_add_weak_pointer (G_OBJECT(msg), (gpointer)info);
 	g_object_weak_ref (G_OBJECT(msg), unblock_free, soup_sess);
 //	g_object_weak_ref (G_OBJECT(soup_sess), unblock_free, soup_sess);
 //	GMainLoop *mainloop = g_main_loop_new (g_main_context_default (), FALSE);
-  //	g_timeout_add (10 * 1000, &conn_mainloop_quit, mainloop);
+//	g_timeout_add (10 * 1000, &conn_mainloop_quit, mainloop);
 	return TRUE;
 }
 
+
+// same stuff as net_get_* but without accumulating headers
+// push all donwloads to a customizable length queue
+gboolean
+download_unblocking(
+	gchar *url,
+	NetStatusCallback cb,
+	gpointer data,
+	gpointer cb2,
+	gpointer cbdata2,
+	guint track,
+	GError **err)
+{
+	SoupMessage *msg;
+	CallbackInfo *info = NULL;
+	SoupSession *soup_sess;
+	gchar *agstr;
+	STNET *stnet;
+
+	soup_sess = soup_session_async_new();
+
+
+#if LIBSOUP_VERSION > 2024000
+	if (rss_soup_jar) {
+		soup_session_add_feature(soup_sess,
+			SOUP_SESSION_FEATURE(rss_soup_jar));
+	}
+#endif
+
+#if (DATASERVER_VERSION >= 2023001)
+	proxify_session(proxy, soup_sess, url);
+#endif
+	if (cb && data) {
+		info = g_new0(CallbackInfo, 1);
+		info->user_cb = cb;
+		info->user_data = data;
+		info->current = 0;
+		info->total = 0;
+	}
+
+	g_signal_connect (soup_sess, "authenticate",
+		G_CALLBACK (authenticate), (gpointer)url);
+#if LIBSOUP_VERSION < 2003000
+	g_signal_connect (soup_sess, "reauthenticate",
+		G_CALLBACK (reauthenticate), (gpointer)url);
+#endif
+
+	/* Queue an async HTTP request */
+	msg = soup_message_new ("GET", url);
+	if (!msg) {
+		g_set_error(err, NET_ERROR, NET_ERROR_GENERIC, "%s",
+				soup_status_get_phrase(2));			//invalid url
+		return FALSE;
+	}
+
+	if (track) {
+		//we want to be able to abort this session by calling
+		//abort_all_soup
+		g_hash_table_insert(rf->session, soup_sess, msg);
+		g_hash_table_insert(rf->abort_session, soup_sess, msg);
+		g_hash_table_insert(rf->key_session, data, soup_sess);
+	}
+
+	agstr = g_strdup_printf("Evolution/%s; Evolution-RSS/%s",
+			EVOLUTION_VERSION_STRING, VERSION);
+#if LIBSOUP_VERSION < 2003000
+	soup_message_add_header (msg->request_headers, "User-Agent",
+		agstr);
+#else
+	soup_message_headers_append (msg->request_headers, "User-Agent",
+		agstr);
+#endif
+	g_free(agstr);
+
+	if (info) {
+		g_signal_connect(G_OBJECT(msg), "got_chunk",
+			G_CALLBACK(got_chunk_cb), info);	//FIXME Find a way to free this maybe weak_ref
+	}
+
+	soup_message_body_set_accumulate (msg->response_body, FALSE);
+	stnet = g_new0(STNET, 1);
+	stnet->ss = soup_sess;
+	stnet->sm = msg;
+	stnet->cb2 = cb2;
+	stnet->cbdata2 = cbdata2;
+	stnet->url = g_strdup(url);
+	g_queue_push_tail (rf->stqueue, stnet);
+	rf->enclist = g_list_append (rf->enclist, g_strdup(url));
+	if (!net_qid)
+		net_qid = g_idle_add((GSourceFunc)net_queue_dispatcher, NULL);
+
+////	g_object_add_weak_pointer (G_OBJECT(msg), (gpointer)info);
+	g_object_weak_ref (G_OBJECT(msg), unblock_free, soup_sess);
+//	g_object_weak_ref (G_OBJECT(soup_sess), unblock_free, soup_sess);
+//	GMainLoop *mainloop = g_main_loop_new (g_main_context_default (), FALSE);
+//	g_timeout_add (10 * 1000, &conn_mainloop_quit, mainloop);
+	return TRUE;
+}
+
+gboolean
+net_queue_dispatcher(void)
+{
+	STNET *_stnet;
+	guint qlen = g_queue_get_length(rf->stqueue);
+
+	d("que len:%d workers:%d\n",
+		g_queue_get_length(rf->stqueue),
+		net_queue_run_count);
+
+	if (qlen && net_queue_run_count < gconf_client_get_int (
+						rss_gconf,
+						GCONF_KEY_DOWNLOAD_QUEUE_SIZE,
+						NULL)) {
+		net_queue_run_count++;
+		_stnet = g_queue_pop_head(rf->stqueue);
+		soup_session_queue_message (
+			_stnet->ss,
+			_stnet->sm,
+			_stnet->cb2,
+			_stnet->cbdata2);
+		g_free(_stnet);
+		return TRUE;
+	}
+	net_qid = 0;
+	return FALSE;
+}
+
 GString*
-net_post_blocking(gchar *url, GSList *headers, GString *post,
-                  NetStatusCallback cb, gpointer data,
-                  GError **err) {
+net_post_blocking(gchar *url,
+	GSList *headers,
+	GString *post,
+	NetStatusCallback cb,
+	gpointer data,
+	GError **err) {
 #if LIBSOUP_VERSION < 2003000
 	SoupUri *suri = NULL;
 #else
@@ -619,28 +800,36 @@ net_post_blocking(gchar *url, GSList *headers, GString *post,
 	GString *response = NULL;
 	CallbackInfo info = { cb, data, 0, 0 };
 	SoupSession *soup_sess = NULL;
+	gchar *agstr;
 
 	if (!rf->b_session)
-		rf->b_session = soup_sess = 
-			soup_session_sync_new_with_options(SOUP_SESSION_TIMEOUT, SS_TIMEOUT, NULL);		
+		rf->b_session = soup_sess =
+			soup_session_sync_new_with_options(
+				SOUP_SESSION_TIMEOUT,
+				SS_TIMEOUT,
+				NULL);
 	else
 		soup_sess = rf->b_session;
 
 
-	g_signal_connect (soup_sess, "authenticate",
-            G_CALLBACK (authenticate), (gpointer)url);
+	g_signal_connect (soup_sess,
+		"authenticate",
+		G_CALLBACK (authenticate),
+		(gpointer)url);
 #if LIBSOUP_VERSION < 2003000
-	g_signal_connect (soup_sess, "reauthenticate",
-            G_CALLBACK (reauthenticate), (gpointer)url);
+	g_signal_connect (soup_sess,
+		"reauthenticate",
+		G_CALLBACK (reauthenticate),
+		(gpointer)url);
 #endif
 
 	req = soup_message_new(SOUP_METHOD_GET, url);
 	if (!req) {
-		g_set_error(err, NET_ERROR, NET_ERROR_GENERIC,
+		g_set_error(err, NET_ERROR, NET_ERROR_GENERIC, "%s",
 				soup_status_get_phrase(2));			//invalid url
 		goto out;
 	}
-	d(g_print("request ok :%d\n", req->status_code));
+	d("request ok :%d\n", req->status_code);
 	g_signal_connect(G_OBJECT(req), "got-chunk",
 			G_CALLBACK(got_chunk_blocking_cb), &info);
 	for (; headers; headers = headers->next) {
@@ -650,20 +839,28 @@ net_post_blocking(gchar *url, GSList *headers, GString *post,
 		char *colonpos = strchr(header, ':');
 		*colonpos = 0;
 #if LIBSOUP_VERSION < 2003000
-		soup_message_add_header(req->request_headers, header, colonpos+1);
+		soup_message_add_header(
+			req->request_headers,
+			header,
+			colonpos+1);
 #else
-		soup_message_headers_append(req->request_headers, header, colonpos+1);
+		soup_message_headers_append(
+			req->request_headers, header, colonpos+1);
 #endif
 		*colonpos = ':';
 	}
-	gchar *agstr = g_strdup_printf("Evolution/%s; Evolution-RSS/%s",
+	agstr = g_strdup_printf("Evolution/%s; Evolution-RSS/%s",
 			EVOLUTION_VERSION_STRING, VERSION);
 #if LIBSOUP_VERSION < 2003000
-	soup_message_add_header (req->request_headers, "User-Agent",
-                                agstr);
+	soup_message_add_header (
+		req->request_headers,
+		"User-Agent",
+		agstr);
 #else
-	soup_message_headers_append (req->request_headers, "User-Agent",
-                                agstr);
+	soup_message_headers_append (
+		req->request_headers,
+		"User-Agent",
+		agstr);
 #endif
 	g_free(agstr);
 
@@ -679,66 +876,75 @@ net_post_blocking(gchar *url, GSList *headers, GString *post,
 		soup_session_abort(soup_sess);
 		g_object_unref(soup_sess);
 		rf->b_session = NULL;
-		g_set_error(err, NET_ERROR, NET_ERROR_GENERIC,
+		g_set_error(err, NET_ERROR, NET_ERROR_GENERIC, "%s",
 				soup_status_get_phrase(req->status_code));
 		goto out;
 	}
 
 #if LIBSOUP_VERSION < 2003000
-	response = g_string_new_len(req->response.body, req->response.length);
+	response = g_string_new_len(
+			req->response.body,
+			req->response.length);
 #else
-	response = g_string_new_len(req->response_body->data, req->response_body->length);
+	response = g_string_new_len(
+			req->response_body->data,
+			req->response_body->length);
 #endif
 
 out:
 	if (suri) soup_uri_free(suri);
 	if (req) g_object_unref(G_OBJECT(req));
-	
+
 	return response;
 }
 
 gboolean
 cancel_soup_sess(gpointer key, gpointer value, gpointer user_data)
 {
-        if (SOUP_IS_SESSION(key)) {
-                soup_session_abort(key);
-                g_hash_table_find(rf->key_session,
-                        remove_if_match,
-                        user_data);
-        }
-        return TRUE;
+	if (SOUP_IS_SESSION(key)) {
+		soup_session_abort(key);
+		g_hash_table_find(rf->key_session,
+			remove_if_match,
+			user_data);
+	}
+	return TRUE;
 }
 
+void remove_weak(gpointer key, gpointer value, gpointer user_data);
+
 void
 remove_weak(gpointer key, gpointer value, gpointer user_data)
 {
-        g_object_weak_unref(value, unblock_free, key);
+	g_object_weak_unref(value, unblock_free, key);
 }
 
 void
 abort_all_soup(void)
 {
-        //abort all session
-        rf->cancel = 1;
-        rf->cancel_all = 1;
-        if (rf->abort_session) {
-                g_hash_table_foreach(rf->abort_session, remove_weak, NULL);
-                g_hash_table_foreach_remove(rf->abort_session, cancel_soup_sess, NULL);
+	//abort all session
+	rf->cancel = 1;
+	rf->cancel_all = 1;
+	if (rf->abort_session) {
+		g_hash_table_foreach(rf->abort_session, remove_weak, NULL);
+		g_hash_table_foreach_remove(
+			rf->abort_session, cancel_soup_sess, NULL);
 //              g_hash_table_foreach(rf->abort_session, cancel_soup_sess, NULL);
-                g_hash_table_destroy(rf->session);
-                rf->session = g_hash_table_new(g_direct_hash, g_direct_equal);
-        }
-        if (rf->progress_bar) {
-                gtk_progress_bar_set_fraction((GtkProgressBar *)rf->progress_bar, 1);
-                rf->progress_bar = NULL;        //there's no need to update bar once we canceled feeds
-        }
-        if (rf->b_session) {
-                soup_session_abort(rf->b_session);
-                rf->b_session = NULL;
-                rf->b_msg_session = NULL;
-        }
+		g_hash_table_destroy(rf->session);
+		rf->session = g_hash_table_new(
+				g_direct_hash, g_direct_equal);
+	}
+	if (rf->progress_bar) {
+		gtk_progress_bar_set_fraction(
+			(GtkProgressBar *)rf->progress_bar, 1);
+		rf->progress_bar = NULL;        //there's no need to update bar once we canceled feeds
+	}
+	if (rf->b_session) {
+		soup_session_abort(rf->b_session);
+		rf->b_session = NULL;
+		rf->b_msg_session = NULL;
+	}
 	rf->cancel = 0;
-        rf->cancel_all = 0;
+	rf->cancel_all = 0;
 }
 
 void
@@ -750,13 +956,17 @@ sync_gecko_cookies(void)
 	//gecko renderer
 	//symlink(cookie_path, moz_cookie_path);
 	GFile *cookie_file, *moz_cookie_file;
-	gchar *feed_dir = rss_component_peek_base_directory(mail_component_peek());
-	gchar *cookie_path = g_build_path("/", feed_dir, "rss-cookies.sqlite", NULL);
-	gchar *moz_cookie_path = g_build_path("/", feed_dir, "mozembed-rss", "cookies.sqlite", NULL);
+	gchar *feed_dir = rss_component_peek_base_directory();
+	gchar *cookie_path = g_build_path(
+				G_DIR_SEPARATOR_S, feed_dir,
+				"rss-cookies.sqlite", NULL);
+	gchar *moz_cookie_path = g_build_path(
+				G_DIR_SEPARATOR_S, feed_dir,
+				"mozembed-rss", "cookies.sqlite", NULL);
 
 	cookie_file = g_file_new_for_path (cookie_path);
 	moz_cookie_file = g_file_new_for_path (moz_cookie_path);
-	g_file_copy(cookie_file, moz_cookie_file, 
+	g_file_copy(cookie_file, moz_cookie_file,
 			G_FILE_COPY_OVERWRITE,
 			NULL, NULL, NULL, NULL);
 	g_free(cookie_path);
@@ -766,13 +976,23 @@ sync_gecko_cookies(void)
 void
 rss_soup_init(void)
 {
-#if LIBSOUP_VERSION > 2026002 && defined(HAVE_LIBSOUP_GNOME) 
+#if LIBSOUP_VERSION > 2026002 && defined(HAVE_LIBSOUP_GNOME)
 	if (gconf_client_get_bool (rss_gconf, GCONF_KEY_ACCEPT_COOKIES, NULL)) {
-		gchar *feed_dir = rss_component_peek_base_directory(mail_component_peek());
-		gchar *cookie_path = g_build_path("/", feed_dir, "rss-cookies.sqlite", NULL);
-		gchar *moz_cookie_path = g_build_path("/", feed_dir, "mozembed-rss", "cookies.sqlite", NULL);
+		gchar *feed_dir = rss_component_peek_base_directory();
+		gchar *cookie_path = g_build_path(
+					G_DIR_SEPARATOR_S,
+					feed_dir,
+					"rss-cookies.sqlite",
+					NULL);
+		gchar *moz_cookie_path = g_build_path(
+						G_DIR_SEPARATOR_S,
+						feed_dir,
+						"mozembed-rss",
+						"cookies.sqlite",
+						NULL);
+		g_free(feed_dir);
 
-		rss_soup_jar = 
+		rss_soup_jar =
 			soup_cookie_jar_sqlite_new (cookie_path, FALSE);
 
 		if (!g_file_test(moz_cookie_path, G_FILE_TEST_EXISTS|G_FILE_TEST_IS_SYMLINK)) {
@@ -782,4 +1002,6 @@ rss_soup_init(void)
 		g_free(moz_cookie_path);
 	}
 #endif
+	if (!rf->stqueue)
+		rf->stqueue = g_queue_new();
 }
